@@ -948,6 +948,10 @@ class Client(SwaggerClient):
 
             # sort to avoid "overlapping columns" error in handsontable's NestedHeaders
             sorted_columns = flatten(unflatten(columns, splitter="dot"), reducer="dot")
+            # also sort by increasing nesting for better columns display
+            sorted_columns = dict(
+                sorted(sorted_columns.items(), key=lambda item: item[0].count("."))
+            )
 
             # reconcile with existing columns
             resp = self.projects.getProjectByName(pk=self.project, _fields=["columns"]).result()
@@ -1523,8 +1527,7 @@ class Client(SwaggerClient):
         require_one_of = {"data"} | set(COMPONENTS)
         per_page = self._get_per_page(per_request)
 
-        for idx, contrib in enumerate(contributions):
-            c = contributions[idx] = unflatten(contrib, splitter="dot")
+        for idx, c in enumerate(contributions):
             has_keys = require_one_of & c.keys()
             if not has_keys:
                 return {"error": f"Nothing to submit for contribution #{idx}!"}
@@ -1575,6 +1578,7 @@ class Client(SwaggerClient):
 
         for contrib in tqdm(contributions, desc="Prepare"):
             if "data" in contrib:
+                contrib["data"] = unflatten(contrib["data"], splitter="dot")
                 serializable, error = self._is_serializable_dict(contrib["data"])
                 if not serializable:
                     raise ValueError(error)
@@ -1708,6 +1712,7 @@ class Client(SwaggerClient):
                 future = self.session.post(
                     f"{self.url}/contributions/",
                     headers=self.headers,
+                    hooks={'response': _response_hook},
                     data=payload,
                 )
                 setattr(future, "track_id", track_id)
@@ -1717,6 +1722,7 @@ class Client(SwaggerClient):
                 future = self.session.put(
                     f"{self.url}/contributions/{pk}/",
                     headers=self.headers,
+                    hooks={'response': _response_hook},
                     data=payload,
                 )
                 setattr(future, "track_id", pk)
